

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

library UNISIM;
use UNISIM.VCOMPONENTS.ALL;


entity plb_sdram_emc_arb is
    generic (
        C_PLB_NUM_MASTERS               : integer          := 4;
        C_PLB_AWIDTH                    : integer          := 32
     );

  port
      (
        -- Clocks and reset
        PLB_Clk             : in  std_logic;
        PLB_Rst             : in  std_logic;

        -- PLB Slave signals
        PLB_ABus        : in  std_logic_vector(0 to C_PLB_AWIDTH-1); --
        PLB_PAValid     : in  std_logic;  --
        PLB_SAValid     : in  std_logic;
        Sl_wait         : out std_logic;                            
		Sl_addrAck      : out  std_logic; --
		Sl_MBusy        : out  std_logic_vector(0 to C_PLB_NUM_MASTERS-1); --
      
		PLB_PAValid_e  : out std_logic; --
		PLB_PAValid_s  : out std_logic; --
        PLB_SAValid_e  : out std_logic;
        PLB_SAValid_s  : out std_logic;
		Sl_wait_s         : in  std_logic; --
		Sl_wait_e         : in  std_logic; --
		Sl_addrAck_s      : in  std_logic; --
		Sl_addrAck_e      : in  std_logic; --
		Sl_MBusy_s        : in  std_logic_vector(0 to C_PLB_NUM_MASTERS-1); --
		Sl_MBusy_e        : in  std_logic_vector(0 to C_PLB_NUM_MASTERS-1) --
);
end plb_sdram_emc_arb;


architecture STRUCTURE of plb_sdram_emc_arb is

CONSTANT  C_BASEADDR   : std_logic_vector(0 to 31) := X"0000_0000";
CONSTANT  C_MEM0_BASEADDR  : std_logic_vector(0 to 31) := X"F0F0_0000";

signal  Sl_MBusy_det_s : std_logic; --
signal  Sl_MBusy_det_e : std_logic; --
signal  adr_det_s  : std_logic; --
signal  adr_det_e  : std_logic; --
signal  adr_det_e_4lc  : std_logic; --
signal  Sl_wait_w  : std_logic; --

constant Sl_MBusy_zero : std_logic_vector(0 to C_PLB_NUM_MASTERS-1) := (others => '0') ; --


begin -- architecture IMP


PLB_PAValid_wait_s_p: process (PLB_Clk) is
begin
if PLB_Clk'event and PLB_Clk = '1' then
	if PLB_Rst = '1' then
		Sl_MBusy_det_s <= '0';
	elsif Sl_MBusy_s /= Sl_MBusy_zero then
		Sl_MBusy_det_s <= '1';
	else
		Sl_MBusy_det_s <= '0';
	end if;
end if;
end process PLB_PAValid_wait_s_p;

PLB_PAValid_e <= PLB_PAValid and not Sl_MBusy_det_s;
PLB_SAValid_e <= PLB_PAValid and not Sl_MBusy_det_s;


PLB_PAValid_wait_e_p: process (PLB_Clk) is
begin
if PLB_Clk'event and PLB_Clk = '1' then
	if PLB_Rst = '1' then
		Sl_MBusy_det_e <= '0';
	elsif Sl_MBusy_e /= Sl_MBusy_zero then
		Sl_MBusy_det_e <= '1';
	else
		Sl_MBusy_det_e <= '0';
	end if;
end if;
end process PLB_PAValid_wait_e_p;

PLB_PAValid_s <= PLB_PAValid and not Sl_MBusy_det_e;
PLB_SAValid_s <= PLB_PAValid and not Sl_MBusy_det_e;



adr_det_s <= '1' when PLB_ABus(0 to 6) = C_BASEADDR(0 to 6) else '0';
adr_det_e <= '1' when PLB_ABus(0 to 7) = C_MEM0_BASEADDR(0 to 7) else '0';
adr_det_e_4lc <= '0' when PLB_ABus(8 to 11) = C_MEM0_BASEADDR(8 to 11) else '1';


Sl_MBusy_w_p: process (PLB_Clk) is
begin
if PLB_Clk'event and PLB_Clk = '1' then
	if PLB_Rst = '1' then
		Sl_wait_w <= '0';
	elsif Sl_addrAck_s = '1' or Sl_addrAck_e = '1' then
		Sl_wait_w <= '0';
	elsif Sl_MBusy_det_s = '1' and adr_det_e = '1' and adr_det_e_4lc = '1' and PLB_PAValid = '1' then
		Sl_wait_w <= '1';
	elsif Sl_MBusy_det_e = '1' and adr_det_s = '1' and PLB_PAValid = '1' then
		Sl_wait_w <= '1';
	end if;
end if;
end process Sl_MBusy_w_p;


Sl_wait <= Sl_wait_w or Sl_wait_s or Sl_wait_e;

Sl_addrAck <= Sl_addrAck_s or Sl_addrAck_e;
Sl_MBusy <= Sl_MBusy_s or Sl_MBusy_s;

end architecture STRUCTURE;